from imp import new_module
from nose.case import Test
from nose.core import TestProgram
from nose.loader import TestLoader
from nose.plugins import PluginTester 
from nose_fixes.plugin import Plugin
from unittest import TestCase, makeSuite

class Tests(TestCase):

    def setUp(self):
        self.plugin = Plugin()
        
    def test_prepareTestResult_sets_descriptions(self):
        thing = object()
        self.plugin.show_docstrings = thing
        class FakeResult(object):
            def __init__(self):
                self.descriptions = None
        result = FakeResult()
        self.plugin.prepareTestResult(result)
        self.assertEqual(result.descriptions, thing)

class FunctionalMixin(PluginTester):

    plugins = [Plugin()]
    activate = ''
    args = ['-v']

    def makeSuite(self):
        class TC(TestCase):
            def test_name0(self):
                'confusing output!'
                pass
            def test_name1(self):
                'confusing output!'
                self.fail()
        return makeSuite(TC)
    
class TestHideDocStrings(FunctionalMixin, TestCase):

    def test(self):
        self.assertFalse('confusing output!' in self.output)
        self.assertTrue('name0' in self.output)
        self.assertTrue('name1' in self.output)

class TestShowDocStrings(FunctionalMixin, TestCase):

    activate = '--show-docstrings'

    def test(self):
        self.assertTrue('confusing output!' in self.output)
        self.assertFalse('name0' in self.output)



# Copyright (c) 2011-2012 Simplistix Ltd
# See license.txt for license details.

import os
from setuptools import setup, find_packages

name = 'nose_fixes'

setup(
    name=name,
    version='1.2',
    author='Chris Withers',
    author_email='chris@simplistix.co.uk',
    license='MIT',
    description="A plugin to make nose behave better.",
    long_description=open(os.path.join(os.path.dirname(__file__),
                                       'docs','description.txt')).read(),
    url='http://packages.python.org/'+name,
    classifiers=[
    'Development Status :: 5 - Production/Stable',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    ],    
    packages=find_packages(),
    zip_safe=False,
    include_package_data=True,
    install_requires=[
        'nose',
        'setuptools', # so nose's EntryPointPluginManager gets used
        ],
    entry_points = {
        'nose.plugins.0.10': [
            'nose_fixes = nose_fixes.plugin:Plugin'
            ]
        },
    )



from imp import new_module
from nose.case import Test as NoseTest
from nose.suite import ContextSuite
from nose.loader import TestLoader
from unittest import TestCase, TestSuite, makeSuite

class Tests(TestCase):

    def setUp(self):
        from nose_fixes.plugin import Plugin
        plugin = Plugin()
        plugin.test_suite_func = 'test_suite'
        self.original_loader = TestLoader()
        self.l = plugin.prepareTestLoader(self.original_loader)
        
    def test_suite(self):

        class Tests(TestCase):
            def test_something(self):
                return

        def test_suite():
            return makeSuite(Tests)
        
        mod = new_module('with_test_suite')
        mod.test_suite = test_suite

        result = self.l.loadTestsFromModule(mod)
        self.assertTrue(isinstance(result, ContextSuite))
        tests = tuple(result)
        self.assertEqual(len(tests), 1)
        test = tests[0]
        self.assertTrue(isinstance(test, NoseTest))
        self.assertTrue(isinstance(test.test, Tests))

    def test_suite_returns_nested(self):

        class Tests(TestCase):
            def test_something(self):
                return

        def test_suite():
            return TestSuite([TestSuite([makeSuite(Tests)])])
        
        mod = new_module('with_test_suite')
        mod.test_suite = test_suite

        result = self.l.loadTestsFromModule(mod)
        self.assertTrue(isinstance(result, ContextSuite))
        tests = tuple(result)
        self.assertEqual(len(tests), 1)
        test = tests[0]
        self.assertTrue(isinstance(test, NoseTest))
        self.assertTrue(isinstance(test.test, Tests))

    def test_suite_returns_something_odd(self):
        class dodgy(object):
            def __repr__(self):
                return '<dodgy>'
        
        def test_suite():
            return [dodgy()]
        
        mod = new_module('with_test_suite')
        mod.test_suite = test_suite

        try:
            self.l.loadTestsFromModule(mod)
        except TypeError, e:
            self.assertEqual(
                str(e),
                "Don't know what to do with <dodgy> "
                "(<class 'nose_fixes.tests.test_collection.dodgy'>)"
                )
        else:
            self.fail('No exception raised!')

    def test_no_suite(self):
        def test_sweet():
            # a tester of sweets ;-)
            assert True!=False
        
        mod = new_module('without_test_suite')
        mod.test_sweet = test_sweet

        suite = self.l.loadTestsFromModule(mod)

        tests = [t for t in suite]
        self.assertEqual(len(tests), 1)
        test = tests[0]
        self.assertTrue(isinstance(test, NoseTest))
        self.assertEqual(repr(test), 'Test(without_test_suite.test_sweet)')

    def test_config_kept(self):
        self.assertTrue(self.l.config is self.original_loader.config)



from nose.case import Test
from nose.plugins import Plugin as NosePlugin
from nose.loader import TestLoader
from unittest import TestCase, TestSuite

class BetterLoader(TestLoader):

    def __init__(self, test_suite_func, config):
        super(BetterLoader, self).__init__(config)
        self.test_suite_func = test_suite_func

    def loadTestsFromModule(self, module, path=None, discovered=False):
        suite_func = getattr(module, self.test_suite_func, None)
        if suite_func is not None:
            to_process = list(suite_func())
            suite = self.suiteClass([])
            for entity in to_process:
                if isinstance(entity, TestCase):
                    suite.addTest(entity)
                elif isinstance(entity, TestSuite):
                    to_process.extend(entity)
                else:
                    raise TypeError(
                        "Don't know what to do with %r (%r)" % (
                            entity, type(entity)
                            ))
            return suite
        
        return super(BetterLoader, self).loadTestsFromModule(
            module, path, discovered
            )
    
class Plugin(NosePlugin):

    name = 'nose_fixes'

    enabled = True
    
    def options(self, parser, env):
        parser.add_option("--test-suite-func", action="store",
                          dest="test_suite_func",
                          default='test_suite',
                          help="A function in modules that will return a TestSuite. "
                               "Defaults to 'test_suite'.")
        parser.add_option('--show-docstrings', dest='show_docstrings',
                          action='store_true', default=False,
                          help='Allow docstrings to be displayed as test '
                          'descriptions.')
    
    def configure(self, options, config):
        self.test_suite_func = options.test_suite_func
        self.show_docstrings = options.show_docstrings
        
    def prepareTestLoader(self, loader):
        return BetterLoader(self.test_suite_func, loader.config)

    def prepareTestResult(self, result):
        result.descriptions = self.show_docstrings
